#include "JniHelper.h"
jobject JniHelper::CreatePoint(jint x, jint y) {
	return env_->NewObject(clsPoint_, ctorPointFromII_, x, y);
}

jobject JniHelper::CreatePoint(jobject point) {
	return env_->NewObject(clsPoint_, ctorPointFromPoint_, point);
}

jint JniHelper::GetPointX(jobject point) {
	return env_->GetIntField(point, fldPointX_);
}
jint JniHelper::GetPointY(jobject point) {
	return env_->GetIntField(point, fldPointY_);
}
void JniHelper::SetPointX(jobject point, jint value) {
	env_->SetIntField(point, fldPointX_, value);
}
void JniHelper::SetPointY(jobject point, jint value) {
	env_->SetIntField(point, fldPointY_, value);
}

jobject JniHelper::CreateCell(jint color, jobject point) {
	if (DEBUG)
		__android_log_print(
				ANDROID_LOG_DEBUG,
				"JNI",
				"JniHelper#CreateCell(). color: %d, color addr: %p, point: %p, clsCell: %p, ctorCell: %p",
				color, &color, point, clsCell_, ctorCell_);
	return env_->NewObject(clsCell_, ctorCell_, color, point);
}

jobjectArray JniHelper::CreateTwoDimensionalCellArray(jint rows, jint columns) {
	jclass clsArray;
	jobjectArray column = (jobjectArray) env_->NewObjectArray((jsize) columns,
			clsCell_, 0);
	clsArray = env_->GetObjectClass((jobject) column);
	jobjectArray cells = env_->NewObjectArray((jsize) rows, clsArray, 0);
	for (jint i = 0; i < rows; i++) {
		env_->SetObjectArrayElement(cells, i,
				env_->NewObjectArray((jsize) columns, clsCell_, 0));
	}
	DeleteLocal(clsArray);
	DeleteLocal(column);
	return cells;
}

jobjectArray JniHelper::CreateOneDimensionalCellArray(jint rows, jint columns) {
	jobjectArray cells = (jobjectArray) env_->NewObjectArray(
			(jsize) (columns * rows), clsCell_, 0);
	return cells;
}

jobject JniHelper::GetElementFromArray(jobjectArray array, jsize index) {
	return env_->GetObjectArrayElement(array, index);
}

void JniHelper::SetElementToArray(jobjectArray array, jsize index,
		jobject element) {
	env_->SetObjectArrayElement(array, index, element);
}

jint JniHelper::CreateColor(jint r, jint g, jint b) {
	return env_->CallStaticIntMethod(clsColor_, mtdColorRgb_, r, g, b);
}

jint JniHelper::GetCellColor(jobject cell) {
	return env_->CallIntMethod(cell, mtdCellGetColor_);
}

jobject JniHelper::GetCellPosition(jobject cell) {
	return env_->CallObjectMethod(cell, mtdCellGetPosition_);
}

jobjectArray JniHelper::GetIShapeCells(jobject shape) {
	return (jobjectArray) env_->CallObjectMethod(shape, mtdIShapeGetCells_);
}

JNIEnv* JniHelper::GetEnv() {
	return env_;
}

void JniHelper::DeleteLocal(jobject object) {
	env_->DeleteLocalRef(object);
}

void JniHelper::DeleteGlobal(jobject object) {
	env_->DeleteGlobalRef(object);
}

void JniHelper::LogString(const char* message) {
	__android_log_print(ANDROID_LOG_DEBUG, "JNI", message);
}
