#ifndef JNIHELPER_H
#define JNIHELPER_H
#include "jni.h"
#include "Constants.h"
#include <android/log.h>
class JniHelper {
public:
	JniHelper(JNIEnv* env) {
		env_ = env;
		clsPoint_ = (jclass) env_->NewGlobalRef(
				env_->FindClass("android/graphics/Point"));
		ctorPointFromII_ = env_->GetMethodID(clsPoint_, "<init>", "(II)V");
		ctorPointFromPoint_ = env_->GetMethodID(clsPoint_, "<init>",
				"(Landroid/graphics/Point;)V");
		fldPointX_ = env_->GetFieldID(clsPoint_, "x", "I");
		fldPointY_ = env_->GetFieldID(clsPoint_, "y", "I");
		clsCell_ = (jclass) env_->NewGlobalRef(
				env_->FindClass("com/vickingur/model/shapes/Cell"));
		ctorCell_ = env_->GetMethodID(clsCell_, "<init>",
				"(ILandroid/graphics/Point;)V");
		mtdCellGetColor_ = env->GetMethodID(clsCell_, "getColor", "()I");
		mtdCellGetPosition_ = env->GetMethodID(clsCell_, "getPosition",
				"()Landroid/graphics/Point;");
		clsColor_ = (jclass) env_->NewGlobalRef(
				env_->FindClass("android/graphics/Color"));
		mtdColorRgb_ = env_->GetStaticMethodID(clsColor_, "rgb", "(III)I");

		clsIShape_ = (jclass) env_->NewGlobalRef(
				env_->FindClass("com/vickingur/model/shapes/IShape"));
		mtdIShapeGetCells_ = env_->GetMethodID(clsIShape_, "getCells",
				"()[Lcom/vickingur/model/shapes/Cell;");
	}
	~JniHelper() {
		if (clsPoint_) {
			DeleteGlobal(clsPoint_);
			clsPoint_ = 0;
		}
		if (clsCell_) {
			DeleteGlobal(clsCell_);
			clsCell_ = 0;
		}
		if (clsColor_) {
			DeleteGlobal(clsColor_);
			clsColor_ = 0;
		}
		if (clsIShape_) {
			DeleteGlobal(clsIShape_);
			clsIShape_ = 0;
		}
	}
	jobject CreatePoint(jint x, jint y);
	jobject CreatePoint(jobject point);
	jobject CreateCell(jint color, jobject point);
	jobjectArray CreateTwoDimensionalCellArray(jint rows, jint columns);
	jobjectArray CreateOneDimensionalCellArray(jint rows, jint columns);
	jobject GetElementFromArray(jobjectArray array, jsize index);
	void SetElementToArray(jobjectArray array, jsize index, jobject element);
	jint GetCellColor(jobject cell);
	jobject GetCellPosition(jobject cell);
	jint CreateColor(jint r, jint g, jint b);
	jint GetPointX(jobject point);
	jint GetPointY(jobject point);
	void SetPointX(jobject point, jint value);
	void SetPointY(jobject point, jint value);
	jobjectArray GetIShapeCells(jobject shape);
	JNIEnv* GetEnv();
	void DeleteLocal(jobject object);
	void DeleteGlobal(jobject object);
	void LogString(const char* message);
private:
	JNIEnv* env_;
	jclass clsPoint_;
	jclass clsCell_;
	jclass clsColor_;
	jclass clsIShape_;
	jmethodID ctorPointFromII_;
	jmethodID ctorPointFromPoint_;
	jmethodID ctorCell_;
	jmethodID mtdColorRgb_;
	jmethodID mtdCellGetColor_;
	jmethodID mtdCellGetPosition_;
	jmethodID mtdIShapeGetCells_;
	jfieldID fldPointX_;
	jfieldID fldPointY_;
};
#endif // JNIHELPER_H
