#include "NativeField.h"

NativeField::NativeField(JniHelper* helper, int rows, int columns) {
	helper_ = helper;
	if (DEBUG)
		helper_->LogString("> NativeField#NativeField");
	rows_ = rows;
	columns_ = columns;
	cells_ = new Cell[rows_ * columns_];Clear
	();
	if (DEBUG)
		helper_->LogString("< NativeField#NativeField");
}

NativeField::~NativeField() {
	if (DEBUG)
		helper_->LogString("> NativeField#~NativeField");
	if (cells_) {
		delete[] cells_;
		cells_ = 0;
	}
}

void NativeField::Clear() {
	if (DEBUG)
		helper_->LogString("> NativeField#Clear");
	for (int row = 0; row < rows_; row++) {
		for (int col = 0; col < columns_; col++) {
			Point point(row, col);
			int color = 0;
			cells_[row * columns_ + col] = Cell(color, point);
//        	SetCellAt(row, col, cell);
		}
	}
	if (DEBUG)
		helper_->LogString("< NativeField#Clear");
}

Cell* NativeField::GetCells() {
	if (DEBUG)
		helper_->LogString(">< NativeField#GetCells");
	return cells_;
}

int NativeField::GetRows() {
	if (DEBUG)
		helper_->LogString(">< NativeField#GetRows");
	return rows_;
}
int NativeField::GetColumns() {
	if (DEBUG)
		helper_->LogString(">< NativeField#GetColumns");
	return columns_;
}

void NativeField::PutShape(jobject shape) {
	if (DEBUG)
		helper_->LogString("> NativeField#PutShape()");
	jobjectArray cells = helper_->GetIShapeCells(shape);
	if (DEBUG)
		helper_->LogString("NativeField#PutShape().1");
	jsize rows = helper_->GetEnv()->GetArrayLength(cells);
	if (DEBUG)
		helper_->LogString("NativeField#PutShape().2");
	for (jsize row = 0; row < rows; row++) {
		jobject cell = helper_->GetElementFromArray(cells, row);
		jobject point = helper_->GetCellPosition(cell);
		int x = helper_->GetPointX(point);
		int y = helper_->GetPointY(point);
		Point po(x, y);
		ValidatePoint(po);
		if (DEBUG)
			helper_->LogString("NativeField#PutShape().3");
		int color = helper_->GetCellColor(cell);
		Cell nativeCell(color, po);
		SetCellAt(po.x, po.y, nativeCell);
		if (DEBUG)
			helper_->LogString("NativeField#PutShape().4");
		helper_->DeleteLocal(cell);
		helper_->DeleteLocal(point);
	}
	helper_->DeleteLocal(cells);
	if (DEBUG)
		helper_->LogString("< NativeField#PutShape()");
}
void NativeField::ValidatePoint(Point& point) {
	if (DEBUG)
		helper_->LogString("> NativeField#ValidatePoint");
	Point& valid = point;
	if (valid.x < 0) {
		valid.x += rows_;
	} else {
		if (valid.x >= rows_) {
			valid.x -= rows_;
		}
	}
	if (valid.y < 0) {
		valid.y += columns_;
	} else {
		if (valid.y >= columns_) {
			valid.y -= columns_;
		}
	}
	if (DEBUG)
		helper_->LogString("< NativeField#ValidatePoint");
//	return valid;
}

Cell& NativeField::GetCellAt(int row, int col) {
	if (DEBUG)
		helper_->LogString(">< NativeField#GetCellAt");
	return cells_[row * columns_ + col];
}

void NativeField::SetCellAt(int row, int col, Cell& cell) {
	if (DEBUG)
		helper_->LogString("> NativeField#SetCellAt");
	if (DEBUG)
		__android_log_print(ANDROID_LOG_DEBUG, "JNI",
				" row: %d, col: %d, cell: %p", row, col, &cell);
	int index = row * columns_ + col;
	if (DEBUG)
		helper_->LogString("> NativeField#SetCellAt. assign");
	cells_[index] = cell;
	if (DEBUG)
		helper_->LogString("< NativeField#SetCellAt");
}
