#include "NativeGameProcessor.h"

int NativeGameProcessor::Tick() {
	if (DEBUG)
		helper_->LogString("> NativeGameProcessor#Tick()");
	if (paused_) {
		return alive_;
	}
	int count = 0;
	int rows = field_->GetRows();
	int columns = field_->GetColumns();
	CellAction ca[rows][columns];

	for (int row = 0; row < rows; row++) {
		for (int col = 0; col < columns; col++) {
			Point point(row, col);
			if (AnalyzeCell(point) == DIE) {
				ca[row][col] = DIE;
			} else {
				ca[row][col] = LIVE;
			}
		}
	}
	if (DEBUG)
		helper_->LogString("> NativeGameProcessor#Tick(). Done Analyzing");
	for (int row = 0; row < rows; row++) {
		for (int col = 0; col < columns; col++) {
			Point point(row, col);
			int color = WHITE;
			if (ca[row][col] == LIVE) {
				color = colors_[point];
				count++;
			}
			Cell cell(color, point);
			field_->SetCellAt(row, col, cell);
		}
	}
	alive_ = count;
	generationCount_++;
	colors_.clear();
	if (DEBUG)
		helper_->LogString("< NativeGameProcessor#Tick()");
	return count;
}
void NativeGameProcessor::Clear() {
	if (DEBUG)
		helper_->LogString("> NativeGameProcessor#Clear()");
	field_->Clear();
	generationCount_ = 0;
	paused_ = false;
	if (DEBUG)
		helper_->LogString("< NativeGameProcessor#Tick()");
}
void NativeGameProcessor::CreateField(JNIEnv* env, int rows, int columns) {
	if (DEBUG)
		helper_->LogString("> NativeGameProcessor#CreateField()");
	helper_ = new JniHelper(env);
	field_ = new NativeField(helper_, rows, columns);
	if (DEBUG)
		helper_->LogString("< NativeGameProcessor#CreateField()");
}

void NativeGameProcessor::PutShape(jobject shape) {
	if (DEBUG)
		helper_->LogString("> NativeGameProcessor#PutShape()");
	field_->PutShape(shape);
	if (DEBUG)
		helper_->LogString("< NativeGameProcessor#PutShape()");
}
NativeField* NativeGameProcessor::GetField() {
	if (DEBUG)
		helper_->LogString(">< NativeGameProcessor#GetField()");
	return field_;
}
void NativeGameProcessor::PauseGame() {
	if (DEBUG)
		helper_->LogString("> NativeGameProcessor#PauseGame()");
	paused_ = true;
	if (DEBUG)
		helper_->LogString("< NativeGameProcessor#PauseGame()");
}
void NativeGameProcessor::ResumeGame() {
	if (DEBUG)
		helper_->LogString("> NativeGameProcessor#ResumeGame()");
	paused_ = false;
	if (DEBUG)
		helper_->LogString("< NativeGameProcessor#ResumeGame()");
}
bool NativeGameProcessor::SwitchState() {
	if (DEBUG)
		helper_->LogString("> NativeGameProcessor#SwitchState()");
	paused_ = !paused_;
	if (DEBUG)
		helper_->LogString("< NativeGameProcessor#SwitchState()");
	return paused_;
}
jint NativeGameProcessor::GetGenerationCount() {
	if (DEBUG)
		helper_->LogString(">< NativeGameProcessor#GetGenerationCount()");
	return generationCount_;
}

CellAction NativeGameProcessor::AnalyzeCell(Point& point) {
	if (DEBUG)
		helper_->LogString("> NativeGameProcessor#AnalyzeCell()");
	CellAction action = DIE;
	int numAlive = CalcAliveNeighbours(point);
	int row = point.x;
	int col = point.y;
	Cell& cell = field_->GetCellAt(row, col);
	if (AnalyzeCellState(cell) == ALIVE) {
		if ((numAlive < (int) STARVATION_NUMBER)
				|| (numAlive >= (int) OVERPOPULATION_NUMBER)) {
			action = DIE;
		} else {
			action = LIVE;
			colors_[point] = cell.GetColor();
		}
	} else {
		if (numAlive == 3) {
			action = BORN;
			int color = GetDominatingNeighbouringColor(point);
			colors_[point] = color;
		}
	}
	if (DEBUG)
		helper_->LogString("< NativeGameProcessor#AnalyzeCell()");
	return action;
}

CellState NativeGameProcessor::AnalyzeCellState(Cell& cell) {
	if (DEBUG)
		helper_->LogString("> NativeGameProcessor#AnalyzeCellState()");
	if (cell.GetColor() == (int) WHITE) {
		return DEAD;
	}
	if (DEBUG)
		helper_->LogString("< NativeGameProcessor#AnalyzeCellState()");
	return ALIVE;
}
int NativeGameProcessor::CalcAliveNeighbours(const Point& point) {
	if (DEBUG)
		helper_->LogString("> NativeGameProcessor#CalcAliveNeighbours()");
	int num = 0;
	Point points[8];
	points[0] = Point(point.x - 1, point.y);
	points[1] = Point(point.x + 1, point.y);
	points[2] = Point(point.x - 1, point.y - 1);
	points[3] = Point(point.x + 1, point.y - 1);
	points[4] = Point(point.x - 1, point.y + 1);
	points[5] = Point(point.x + 1, point.y + 1);
	points[6] = Point(point.x, point.y - 1);
	points[7] = Point(point.x, point.y + 1);

	for (int i = 0; i < 8; i++) {
		if (DEBUG)
			__android_log_print(
					ANDROID_LOG_DEBUG,
					"JNI",
					"NativeGameProcessor#CalcAliveNeighbours. i: %d, point.x: %d, point.y: %d",
					i, points[i].x, points[i].y);
		field_->ValidatePoint(points[i]);
		if (DEBUG)
			__android_log_print(
					ANDROID_LOG_DEBUG,
					"JNI",
					"NativeGameProcessor#CalcAliveNeighbours. valid.x: %d, valid.y: %d",
					points[i].x, points[i].y);
		Cell& cell = field_->GetCellAt(points[i].x, points[i].y);
		if (AnalyzeCellState(cell) == ALIVE) {
			num++;
		}
	}
	if (DEBUG)
		helper_->LogString("< NativeGameProcessor#CalcAliveNeighbours()");
	return num;
}
int NativeGameProcessor::GetDominatingNeighbouringColor(Point& point) {
	if (DEBUG)
		helper_->LogString(
				"> NativeGameProcessor#GetDominatingNeighbouringColor()");
	std::map<int, int> colorCount;
	colorCount.clear();
	Point points[8];
	points[0] = Point(point.x - 1, point.y);
	points[1] = Point(point.x + 1, point.y);
	points[2] = Point(point.x - 1, point.y - 1);
	points[3] = Point(point.x + 1, point.y - 1);
	points[4] = Point(point.x - 1, point.y + 1);
	points[5] = Point(point.x + 1, point.y + 1);
	points[6] = Point(point.x, point.y - 1);
	points[7] = Point(point.x, point.y + 1);

	for (int i = 0; i < 8; i++) {
		field_->ValidatePoint(points[i]);
		Cell& cell = field_->GetCellAt(points[i].x, points[i].y);
		int color = cell.GetColor();
		if (color != WHITE) {
			std::map<int, int>::iterator iter = colorCount.find(color);
			if (iter != colorCount.end()) {
				int val = colorCount[color];
				colorCount[color] = val + 1;
			} else {
				colorCount[color] = 1;
			}
		}
	}
	int domColor = -1;
	int max = -1;
	for (std::map<int, int>::iterator iter = colorCount.begin();
			iter != colorCount.end(); ++iter) {
		if (max < iter->second) {
			max = iter->second;
			domColor = iter->first;
		}
	}
	if (DEBUG)
		helper_->LogString(
				"< NativeGameProcessor#GetDominatingNeighbouringColor()");
	return domColor;
}
