#ifndef NATIVEGAMEPROCESSOR_H
#define NATIVEGAMEPROCESSOR_H
#include <stdio.h>
#include <map>
#include <jni.h>
#include "NativeField.h"
#include "Singleton.h"
#include "Constants.h"
enum CellAction {
	BORN, DIE, LIVE
};
enum CellState {
	ALIVE, DEAD
};
class NativeGameProcessor: public Singleton<NativeGameProcessor> {
public:
	NativeGameProcessor() {
		paused_ = false;
		generationCount_ = 0;
		alive_ = 0;
	}
	~NativeGameProcessor() {
		if (field_) {
			delete field_;
			field_ = 0;
		}
		if (helper_) {
			delete helper_;
			helper_ = 0;
		}
	}
	int Tick();
	void Clear();
	void CreateField(JNIEnv* env, int rows, int columns);
	void PutShape(jobject shape);
	NativeField* GetField();
	void PauseGame();
	void ResumeGame();
	bool SwitchState();
	int GetGenerationCount();
private:
	CellAction AnalyzeCell(Point& point);
	CellState AnalyzeCellState(Cell& cell);
	int CalcAliveNeighbours(const Point& point);
	int GetDominatingNeighbouringColor(Point& point);
	NativeField *field_;
	bool paused_;
	int generationCount_;
	int alive_;
	JniHelper* helper_;std::map<Point, int> colors_;
};
#endif // NATIVEGAMEPROCESSOR_H
