#ifndef SINGLETON_H_
#define SINGLETON_H_
#include <pthread.h>
/*! \class Singleton
 *  \brief класс где-то подсмотрен. представляет собой шаблон проектирования синглтон.
 * 		   контролирует, чтобы в системе был только один экземпляр класса
 *  \author Попутников В.В.
 */
template<class T>
class Singleton {
public:
	virtual ~Singleton() {
		pthread_mutex_unlock(&mutex);
	}
	static T *Instance() {
		if (instance == 0) {
			pthread_mutex_lock(&mutex);
			if (instance == 0) {
				instance = new T();
			}
		}
		return instance;
	}
private:
	static T *instance;
	static pthread_mutex_t mutex;
protected:
	Singleton() {
	}

};
template<class T>
T *Singleton<T>::instance = 0;
template<class T>
pthread_mutex_t Singleton<T>::mutex = PTHREAD_MUTEX_INITIALIZER;
#endif /*SINGLETON_H_*/
