#include <string.h>
#include <jni.h>
#include "NativeGameProcessor.h"
#include "com_vickingur_model_NativeGameProcessor.h"
/*
 * Class:     com_vickingur_model_NativeGameProcessor
 * Method:    nativeCreateField
 * Signature: (II)V
 */
JNIEXPORT void JNICALL Java_com_vickingur_model_NativeGameProcessor_nativeCreateField(
		JNIEnv *env, jobject obj, jint rows, jint columns) {
	NativeGameProcessor::Instance()->CreateField(env, (int) rows,
			(int) columns);
}

/*
 * Class:     com_vickingur_model_NativeGameProcessor
 * Method:    nativeTick
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_com_vickingur_model_NativeGameProcessor_nativeTick(
		JNIEnv *env, jobject obj) {
	return NativeGameProcessor::Instance()->Tick();
}

/*
 * Class:     com_vickingur_model_NativeGameProcessor
 * Method:    nativePauseGame
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_com_vickingur_model_NativeGameProcessor_nativePauseGame(
		JNIEnv *env, jobject obj) {
	NativeGameProcessor::Instance()->PauseGame();
}

/*
 * Class:     com_vickingur_model_NativeGameProcessor
 * Method:    nativeResumeGame
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_com_vickingur_model_NativeGameProcessor_nativeResumeGame(
		JNIEnv *env, jobject obj) {
	NativeGameProcessor::Instance()->ResumeGame();
}

/*
 * Class:     com_vickingur_model_NativeGameProcessor
 * Method:    nativeClear
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_com_vickingur_model_NativeGameProcessor_nativeClear(
		JNIEnv *env, jobject obj) {
	NativeGameProcessor::Instance()->Clear();
}

/*
 * Class:     com_vickingur_model_NativeGameProcessor
 * Method:    nativeGetGenerationCount
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_com_vickingur_model_NativeGameProcessor_nativeGetGenerationCount(
		JNIEnv *env, jobject obj) {
	return NativeGameProcessor::Instance()->GetGenerationCount();
}

/*
 * Class:     com_vickingur_model_NativeGameProcessor
 * Method:    nativeSwitchState
 * Signature: ()Z
 */
JNIEXPORT jboolean JNICALL Java_com_vickingur_model_NativeGameProcessor_nativeSwitchState(
		JNIEnv *, jobject) {
	return (jboolean) NativeGameProcessor::Instance()->SwitchState();
}

/*
 * Class:     com_vickingur_model_NativeGameProcessor
 * Method:    nativePutShape
 * Signature: (Lcom/vickingur/model/shapes/IShape;)V
 */
JNIEXPORT void JNICALL Java_com_vickingur_model_NativeGameProcessor_nativePutShape(
		JNIEnv *env, jobject obj, jobject shape) {
	NativeGameProcessor::Instance()->PutShape(shape);
}

/*
 * Class:     com_vickingur_model_NativeGameProcessor
 * Method:    nativeGetCells
 * Signature: ()[[Lcom/vickingur/model/shapes/Cell;
 */
JNIEXPORT jobjectArray JNICALL Java_com_vickingur_model_NativeGameProcessor_nativeGetCells(
		JNIEnv *env, jobject obj) {
	Cell* nativeCells = NativeGameProcessor::Instance()->GetField()->GetCells();
	int rows = NativeGameProcessor::Instance()->GetField()->GetRows();
	int columns = NativeGameProcessor::Instance()->GetField()->GetColumns();
	JniHelper* helper = new JniHelper(env);
	if (DEBUG)
		helper->LogString("> nativeGetCells");
	jobjectArray cells = helper->CreateOneDimensionalCellArray((jint) rows,
			(jint) columns);
	for (int i = 0; i < columns * rows; i++) {
		if (DEBUG)
			__android_log_print(ANDROID_LOG_DEBUG, "JNI", "i: %d", i);
		jint color = (jint) nativeCells[i].GetColor();
		if (DEBUG)
			__android_log_print(ANDROID_LOG_DEBUG, "JNI", "color: %d",
					(int) color);
		jobject point = helper->CreatePoint(nativeCells[i].GetPosition().x,
				nativeCells[i].GetPosition().y);
		if (DEBUG)
			__android_log_print(ANDROID_LOG_DEBUG, "JNI", "pos: %p", point);
		jobject cell = helper->CreateCell(color, point);
		if (DEBUG)
			__android_log_print(ANDROID_LOG_DEBUG, "JNI", "cell: %p", cell);
		helper->SetElementToArray(cells, i, cell);
		if (DEBUG)
			__android_log_print(ANDROID_LOG_DEBUG, "JNI", "deleting");
		helper->DeleteLocal(point);
		helper->DeleteLocal(cell);
	}
	delete helper;
	if (DEBUG)
		helper->LogString("< nativeGetCells");
	return cells;
}

/*
 * Class:     com_vickingur_model_NativeGameProcessor
 * Method:    nativeGetRows
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_com_vickingur_model_NativeGameProcessor_nativeGetRows(
		JNIEnv *env, jobject obj) {
	return NativeGameProcessor::Instance()->GetField()->GetRows();
}

/*
 * Class:     com_vickingur_model_NativeGameProcessor
 * Method:    nativeGetColumns
 * Signature: ()I
 */
JNIEXPORT jint JNICALL Java_com_vickingur_model_NativeGameProcessor_nativeGetColumns(
		JNIEnv *env, jobject obj) {
	return NativeGameProcessor::Instance()->GetField()->GetColumns();
}

/*
 * Class:     com_vickingur_model_NativeGameProcessor
 * Method:    nativeCleanup
 * Signature: ()V
 */
JNIEXPORT void JNICALL Java_com_vickingur_model_NativeGameProcessor_nativeCleanup(
		JNIEnv *, jobject) {
	delete NativeGameProcessor::Instance();
	if (DEBUG)
		__android_log_print(ANDROID_LOG_DEBUG, "JNI",
				"deleted native game processor instance");
}
