package com.vickingur;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Point;
import android.util.Log;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

import com.vickingur.model.GameProcessorFactory;
import com.vickingur.util.Constants;

public class FieldView extends SurfaceView implements SurfaceHolder.Callback {

    private Bitmap bitmap;
    private PainterThread painterThread;

    public FieldView(Context context) {
        super(context);
        getHolder().addCallback(this);
    }

    public PainterThread getThread() {
        if (painterThread == null) {
            painterThread = new PainterThread(getHolder(), getContext());
        }
        return painterThread;
    }

    public void onWindowFocusChanged(boolean hasWindowFocus) {
        if (!hasWindowFocus) {
            getThread().freeze();
        } else {
            getThread().activate();
        }
    }

    public void surfaceChanged(SurfaceHolder holder, int format, int width,
            int height) {
        Log.v("FieldView", "surface changed");
        int w = (int) (Constants.BITMAP_WIDTH_MULTIPLIER * width);
        int h = (int) (Constants.BITMAP_HEIGHT_MULTIPLIER * height);
        getThread().setTopLeft(new Point(width - w, height - h));

        if (bitmap == null) {
            bitmap = Bitmap.createBitmap(w, h, Bitmap.Config.ARGB_8888);
            getThread().setBitmap(bitmap, true);
        }

        getThread().activate();
    }

    public void surfaceCreated(SurfaceHolder holder) {
        getThread().on();
        getThread().start();
    }

    public void surfaceDestroyed(SurfaceHolder holder) {
        boolean retry = true;
        getThread().off();
        while (retry) {
            try {
                getThread().join();
                retry = false;
            } catch (InterruptedException e) {
            }
        }
        painterThread = null;
    }

    public boolean onTouchEvent(MotionEvent event) {
        Log.v("FieldView", "onTouchEvent: " + event);
        if (event.getAction() == MotionEvent.ACTION_DOWN) {
            boolean paused = GameProcessorFactory.getInstance(
                    Constants.IS_NATIVE_IMPL).switchState();
            if (paused) {
                getThread().freeze();
            } else {
                getThread().activate();
            }
        }
        return true;
    }

}
