package com.vickingur;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.graphics.Color;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.ImageButton;
import android.widget.LinearLayout;

import com.vickingur.model.GameProcessorFactory;
import com.vickingur.util.Constants;

public class MogolActivity extends Activity {
    private LinearLayout mainLayout;
    private LinearLayout shapeChoicePanel;
    private ImageButton exitButton;
    private ImageButton newGameButton;
    private ImageButton optionsButton;
    private ImageButton helpButton;
    private ImageButton infoButton;

    private LinearLayout menu;
    private FieldView fieldView;
    private boolean isInitialized = false;

    /** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        buildScreen();
    }

    private void buildScreen() {
        if (isInitialized)
            return;
        isInitialized = true;
        // setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        mainLayout = new LinearLayout(this);
        mainLayout.setOrientation(LinearLayout.VERTICAL);
        mainLayout.setTag("mainLayout");

        exitButton = new ImageButton(this);
        // exitButton.setSingleLine(true);
        // exitButton.setText("Exit");
        // exitButton.setTextSize(Constants.TEXT_SIZE);
        exitButton.setBackgroundResource(R.drawable.exit);
        exitButton.setTag("exitButton");
        exitButton.setPadding(0, 0, 0, 0);
        exitButton.setOnClickListener(new OnClickListener() {

            @Override
            public void onClick(final View v) {
                AlertDialog.Builder builder = new AlertDialog.Builder(v
                        .getContext());
                builder.setMessage("Are you sure you want to exit?")
                        .setCancelable(false)
                        .setPositiveButton("Yes",
                                new DialogInterface.OnClickListener() {
                                    public void onClick(DialogInterface dialog,
                                            int id) {
                                        GameProcessorFactory.getInstance(
                                                Constants.IS_NATIVE_IMPL)
                                                .requestCleanUp();
                                        dialog.cancel();
                                    }
                                })
                        .setNegativeButton("No",
                                new DialogInterface.OnClickListener() {
                                    public void onClick(DialogInterface dialog,
                                            int id) {
                                        dialog.cancel();
                                    }
                                });
                AlertDialog alert = builder.create();
                alert.show();
            }
        });

        newGameButton = new ImageButton(this);
        // newGameButton.setSingleLine(true);
        // newGameButton.setText("New Game");
        // newGameButton.setTextSize(Constants.TEXT_SIZE);
        newGameButton.setBackgroundResource(R.drawable.new_file);
        newGameButton.setTag("newGameButton");
        newGameButton.setPadding(0, 0, 0, 0);
        newGameButton.setOnClickListener(new OnClickListener() {

            @Override
            public void onClick(View v) {
                GameProcessorFactory.getInstance(Constants.IS_NATIVE_IMPL)
                        .clear();
            }
        });

        optionsButton = new ImageButton(this);
        // optionsButton.setSingleLine(true);
        // optionsButton.setText("Options");
        // optionsButton.setTextSize(Constants.TEXT_SIZE);
        optionsButton.setBackgroundResource(R.drawable.tools);
        optionsButton.setTag("optionsButton");
        optionsButton.setPadding(0, 0, 0, 0);
        optionsButton.setOnClickListener(new OnClickListener() {

            @Override
            public void onClick(View v) {
                AlertDialog.Builder builder = new AlertDialog.Builder(v
                        .getContext());
                builder.setMessage("No setup options is available yet")
                        .setIcon(R.drawable.info)
                        .setCancelable(true)
                        .setNeutralButton("OK",
                                new DialogInterface.OnClickListener() {

                                    @Override
                                    public void onClick(DialogInterface dialog,
                                            int which) {
                                        dialog.cancel();
                                    }
                                });
                AlertDialog alert = builder.create();
                alert.show();
            }
        });

        helpButton = new ImageButton(this);
        helpButton.setBackgroundResource(R.drawable.help_book);
        // helpButton.setSingleLine(true);
        // helpButton.setText("Help");
        // helpButton.setTextSize(Constants.TEXT_SIZE);
        helpButton.setTag("helpButton");
        helpButton.setPadding(0, 0, 0, 0);
        helpButton.setOnClickListener(new OnClickListener() {

            @Override
            public void onClick(View v) {
                AlertDialog.Builder builder = new AlertDialog.Builder(v
                        .getContext());
                builder.setMessage("No Help page is available yet")
                        .setIcon(R.drawable.info)
                        .setCancelable(true)
                        .setNeutralButton("OK",
                                new DialogInterface.OnClickListener() {

                                    @Override
                                    public void onClick(DialogInterface dialog,
                                            int which) {
                                        dialog.cancel();
                                    }
                                });
                AlertDialog alert = builder.create();
                alert.show();
            }
        });

        infoButton = new ImageButton(this);
        infoButton.setBackgroundResource(R.drawable.info);
        infoButton.setTag("infoButton");
        infoButton.setPadding(0, 0, 0, 0);
        infoButton.setOnClickListener(new OnClickListener() {

            @Override
            public void onClick(View v) {
                AlertDialog.Builder builder = new AlertDialog.Builder(v
                        .getContext());
                builder.setMessage(
                        "Credits\nVictor Poputnikov\nvickingur@gmail.com\nGreat thanks to my beloved wife and daughter who's inspiring me in my work")
                        .setIcon(R.drawable.info)
                        .setCancelable(true)
                        .setNeutralButton("OK",
                                new DialogInterface.OnClickListener() {

                                    @Override
                                    public void onClick(DialogInterface dialog,
                                            int which) {
                                        dialog.cancel();
                                    }
                                });
                AlertDialog alert = builder.create();
                alert.show();
            }
        });

        menu = new LinearLayout(this);
        menu.setOrientation(LinearLayout.HORIZONTAL);
        menu.setBackgroundColor(Color.DKGRAY);
        menu.setTag("menu");

        LinearLayout.LayoutParams cellLp = new LinearLayout.LayoutParams(
                ViewGroup.LayoutParams.WRAP_CONTENT,
                ViewGroup.LayoutParams.WRAP_CONTENT, 0.2f);

        menu.addView(newGameButton, cellLp);
        menu.addView(optionsButton, cellLp);
        menu.addView(helpButton, cellLp);
        menu.addView(infoButton, cellLp);
        menu.addView(exitButton, cellLp);

        shapeChoicePanel = new PanelView(this);
        shapeChoicePanel.setOrientation(LinearLayout.HORIZONTAL);
        shapeChoicePanel.setBackgroundColor(Color.DKGRAY);
        shapeChoicePanel.setTag("shapeChoicePanel");

        fieldView = new FieldView(this);
        fieldView.setTag("fieldView");

        LinearLayout.LayoutParams lineLp = new LinearLayout.LayoutParams(
                ViewGroup.LayoutParams.FILL_PARENT,
                ViewGroup.LayoutParams.WRAP_CONTENT, 0.1f);
        mainLayout.addView(menu, lineLp);
        mainLayout.addView(shapeChoicePanel, lineLp);
        LinearLayout.LayoutParams fieldLp = new LinearLayout.LayoutParams(
                ViewGroup.LayoutParams.WRAP_CONTENT,
                ViewGroup.LayoutParams.FILL_PARENT, 0.9f);
        mainLayout.addView(fieldView, fieldLp);

        setContentView(mainLayout);
    }
}
