package com.vickingur;

import java.io.IOException;
import java.io.InputStream;

import android.content.Context;
import android.graphics.Point;
import android.util.Log;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.view.inputmethod.InputMethodManager;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.Spinner;
import android.widget.TextView;

import com.vickingur.model.GameProcessorFactory;
import com.vickingur.model.ShapeFactory;
import com.vickingur.model.shapes.IShape;
import com.vickingur.util.Constants;

public class PanelView extends LinearLayout {

    private TextView textViewRow;
    private TextView textViewColumn;
    private EditText textEditRow;
    private EditText textEditColumn;
    private ImageButton putShapeButton;
    private static int WIDGET_HEIGHT = 20;

    public PanelView(Context context) {
        super(context);
        init();
    }

    private Spinner spinner;

    private void init() {
        int filesNum = 0;
        String[] files = null;
        try {
            files = getContext().getAssets().list("");
            // files =
            // getContext().getAssets().list(Constants.IMPORT_SHAPES_DIR+"/");
            filesNum = files.length;
        } catch (IOException e) {
            // TODO Auto-generated catch block
            Log.e("PanelView", "ioexception", e);
        }
        Log.v("PanelView", "num files: " + filesNum);

        // LinearLayout.LayoutParams cellLp = new LinearLayout.LayoutParams(
        // ViewGroup.LayoutParams.WRAP_CONTENT,
        // ViewGroup.LayoutParams.WRAP_CONTENT, 1.0f);

        setPadding(0, 0, 0, 0);

        spinner = new Spinner(getContext());
        ArrayAdapter<CharSequence> adapter = new ArrayAdapter<CharSequence>(
                getContext(), android.R.layout.simple_spinner_item);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        spinner.setAdapter(adapter);

        for (int i = 0; i < filesNum; i++) {
            int index = files[i].indexOf('.');
            if (index > 0) {
                String string = files[i].substring(0, index);
                adapter.add(string);
            }
        }

        textViewRow = new TextView(getContext());
        textViewRow.setTextSize(Constants.TEXT_SIZE);
        textViewRow.setText("Row: ");
        textViewRow.setHeight(WIDGET_HEIGHT);
        textViewRow.setGravity(Gravity.CENTER);
        textViewColumn = new TextView(getContext());
        textViewColumn.setText("Col: ");
        textViewColumn.setTextSize(Constants.TEXT_SIZE);
        textViewColumn.setHeight(WIDGET_HEIGHT);
        textViewColumn.setGravity(Gravity.CENTER);
        textEditRow = new EditText(getContext());
        textEditRow.setText("0");
        textEditRow.setTextSize(Constants.TEXT_SIZE);
        textEditRow.setHeight(WIDGET_HEIGHT);
        // textEditRow.setInputType(0);
        textEditColumn = new EditText(getContext());
        textEditColumn.setText("0");
        textEditColumn.setTextSize(Constants.TEXT_SIZE);
        textEditColumn.setHeight(WIDGET_HEIGHT);
        // textEditColumn.setInputType(0);
        putShapeButton = new ImageButton(getContext());
        // putShapeButton.setText("Put");
        // putShapeButton.setTextSize(Constants.TEXT_SIZE);
        // putShapeButton.setHeight(WIDGET_HEIGHT);
        putShapeButton.setBackgroundResource(R.drawable.record);
        putShapeButton.setOnClickListener(new OnClickListener() {

            @Override
            public void onClick(View v) {
                int x = 0;
                int y = 0;
                try {
                    x = Integer.valueOf(textEditRow.getText().toString());
                    y = Integer.valueOf(textEditColumn.getText().toString());
                } catch (NumberFormatException nfe) {
                    Log.e("PanelView", "wrong input: ", nfe);
                    return;
                }
                String name = (String) spinner.getSelectedItem();
                InputStream is = null;
                try {
                    is = v.getContext()
                            .getAssets()
                            .open(Constants.IMPORT_SHAPES_DIR + "/" + name
                                    + Constants.IMPORT_FILE_EXT);
                } catch (IOException e) {
                    Log.e("PanelView", "IOException for file: "
                            + Constants.IMPORT_TEST_NAME, e);
                }
                IShape imported = ShapeFactory.createNewShape(
                        IShape.ImportedShape, is);
                imported.build(new Point(x, y));
                GameProcessorFactory.getInstance(Constants.IS_NATIVE_IMPL)
                        .addShapeToQ(imported);
                InputMethodManager imm = (InputMethodManager) v.getContext()
                        .getSystemService(Context.INPUT_METHOD_SERVICE);
                imm.hideSoftInputFromWindow(v.getWindowToken(), 0);
            }
        });
        setGravity(Gravity.CENTER);

        LinearLayout.LayoutParams spinnerLp = new LinearLayout.LayoutParams(
                ViewGroup.LayoutParams.FILL_PARENT,
                ViewGroup.LayoutParams.WRAP_CONTENT, 0.3f);

        addView(spinner, spinnerLp);
        addView(textViewRow);
        addView(textEditRow);
        addView(textViewColumn);
        addView(textEditColumn);
        addView(putShapeButton);
    }

}
