package com.vickingur.model;

import java.util.Random;

import android.graphics.Color;
import android.graphics.Point;

import com.vickingur.model.shapes.Cell;
import com.vickingur.model.shapes.IShape;
import com.vickingur.util.Constants;

public class Field {
    private Cell[] cells;
    private int columns;
    private int rows;

    public Field(int rows, int columns) {
        this.rows = rows;
        this.columns = columns;
        cells = new Cell[rows * columns];
        clear();
    }

    public void clear() {
        for (int row = 0; row < rows; row++) {
            for (int col = 0; col < columns; col++) {
                cells[row * columns + col] = new Cell(Color.WHITE, new Point(
                        row, col));
            }
        }
    }

    public void generateRandomField() {
        Random r = new Random();
        for (int row = 0; row < rows; row++) {
            for (int col = 0; col < columns; col++) {
                cells[row * columns + col] = new Cell(Color.rgb(
                        r.nextInt(Constants.RGB_MAX_VALUE),
                        r.nextInt(Constants.RGB_MAX_VALUE),
                        r.nextInt(Constants.RGB_MAX_VALUE)),
                        new Point(row, col));
            }
        }
    }

    public Cell[] getCells() {
        return cells;
    }

    public int getColumns() {
        return columns;
    }

    public int getRows() {
        return rows;
    }

    public void putShape(IShape shape) {
        for (Cell cell : shape.getCells()) {
            Point p = validatePoint(cell.getPosition(), getRows(), getColumns());
            cells[p.x * columns + p.y] = cell;
        }
    }

    public static Point validatePoint(Point point, int rows, int columns) {
        Point p = new Point(point);
        if (p.x < 0) {
            p.x += rows;
        } else {
            if (p.x >= rows) {
                p.x -= rows;
            }
        }
        if (p.y < 0) {
            p.y += columns;
        } else {
            if (p.y >= columns) {
                p.y -= columns;
            }
        }
        return p;
    }

    public int getAliveCellsCount() {
        int count = 0;
        for (int row = 0; row < rows; row++) {
            for (int col = 0; col < columns; col++) {
                if (cells[row * columns + col].getColor() != Color.WHITE) {
                    count++;
                }
            }
        }
        return count;
    }
}
