package com.vickingur.model;

import java.util.LinkedList;
import java.util.Queue;

import com.vickingur.model.shapes.Cell;
import com.vickingur.model.shapes.IShape;

public class NativeGameProcessor implements IGameProcessor {

    static {
        System.loadLibrary("NativeGameProcessor");
    }

    private Queue<IShape> shapeQ = new LinkedList<IShape>();
    private int rows;
    private int columns;
    private boolean needToCleanUpAndStop = false;

    @Override
    public void createField(int rows, int columns) {
        this.rows = rows;
        this.columns = columns;
        nativeCreateField(rows, columns);
    }

    @Override
    public synchronized int tick() {
        if (!isNeedToCleanUpAndStop()) {
            while (!shapeQ.isEmpty()) {
                IShape shape = shapeQ.poll();
                putShape(shape);
            }
            return nativeTick();
        } else {
            nativeCleanup();
        }
        return -1;
    }

    @Override
    public synchronized void pauseGame() {
        nativePauseGame();
    }

    @Override
    public synchronized void resumeGame() {
        nativeResumeGame();
    }

    @Override
    public synchronized void clear() {
        nativeClear();
    }

    @Override
    public synchronized int getGenerationCount() {
        return nativeGetGenerationCount();
    }

    @Override
    public synchronized boolean switchState() {
        return nativeSwitchState();
    }

    @Override
    public synchronized void putShape(IShape shape) {
        nativePutShape(shape);
    }

    @Override
    public synchronized Cell[] getCells() {
        return nativeGetCells();
        // Cell[] cellsOne = nativeGetCells();
        // Cell[][] cellsTwo = new Cell[rows][columns];
        // for(int row = 0; row < rows; row++) {
        // for(int col = 0; col < columns; col++) {
        // cellsTwo[row][col] = cellsOne[row*columns+col];
        // }
        // }
        // return cellsTwo;
    }

    @Override
    public int getRows() {
        return nativeGetRows();
    }

    @Override
    public int getColumns() {
        return nativeGetColumns();
    }

    private native void nativeCreateField(int rows, int columns);

    private native int nativeTick();

    private native void nativePauseGame();

    private native void nativeResumeGame();

    private native void nativeClear();

    private native int nativeGetGenerationCount();

    private native boolean nativeSwitchState();

    private native void nativePutShape(IShape shape);

    private native Cell[] nativeGetCells();

    private native int nativeGetRows();

    private native int nativeGetColumns();

    @Override
    public void addShapeToQ(IShape shape) {
        shapeQ.add(shape);
    }

    @Override
    public void requestCleanUp() {
        needToCleanUpAndStop = true;
    }

    private native void nativeCleanup();

    public boolean isNeedToCleanUpAndStop() {
        return needToCleanUpAndStop;
    }

}
