package com.vickingur.model.shapes;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Random;

import android.graphics.Color;
import android.graphics.Point;
import android.util.Log;

import com.vickingur.util.Constants;

public class ImportedShape extends Shape {

    ArrayList<String> contents;
    private InputStream is;

    public ImportedShape(InputStream is) {
        super();
        this.is = is;
        loadFile();
    }

    private void loadFile() {
        contents = new ArrayList<String>();
        Log.v("ImportedShape", "loadFile. start reading " + name);

        BufferedReader reader = new BufferedReader(new InputStreamReader(is));
        String text = null;
        try {
            while ((text = reader.readLine()) != null) {
                Log.v("ImportedShape", "readLine: " + text);
                String searchString = "Name: ";
                int index = text.indexOf(searchString);
                if (index > 0) {
                    name = text.substring(index + searchString.length());
                } else if (!text.startsWith(Constants.COMMENT_SYMBOL)
                        && !text.equals("")) {
                    Log.v("ImportedShape", "adding line to contents. line: "
                            + text);
                    contents.add(text);
                }
            }
        } catch (IOException e) {
            Log.e("ImportedShape", "IOException caught in file: " + name, e);
        }
    }

    @Override
    public void build(Point start) {
        Random r = new Random();
        int color = Color.rgb(r.nextInt(Constants.RGB_MAX_VALUE),
                r.nextInt(Constants.RGB_MAX_VALUE),
                r.nextInt(Constants.RGB_MAX_VALUE));
        Log.v("ImportedShape",
                "build. color: " + color + " rows: " + contents.size());
        ArrayList<Cell> tempCells = new ArrayList<Cell>();
        for (int row = 0; row < contents.size(); row++) {
            String line = contents.get(row);
            Log.v("ImportedShape",
                    "build. rowNum: " + row + " cols: " + line.length()
                            + " rowText: " + line);
            for (int col = 0; col < line.length(); col++) {
                if (line.charAt(col) == Constants.CELL_SYMBOL) {
                    Log.v("ImportedShape", "adding cell at (" + row + "," + col
                            + ")");
                    tempCells.add(new Cell(color, new Point(start.x + row,
                            start.y + col)));
                }
            }
        }
        cells = new Cell[tempCells.size()];
        int i = 0;
        for (Cell c : tempCells) {
            cells[i] = c;
            i++;
        }
    }

}
