package com.vickingur.util;

import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.Point;
import android.util.Log;

import com.vickingur.model.shapes.Cell;

public class DrawUtil {
    public static void drawCircleCell(Cell cell, Canvas canvas,
            float cellWidth, float cellHeight, Paint paint, Point topLeft) {
        int color = cell.getColor();
        if (color != Color.WHITE) {
            paint.setColor(color);
            float cx = topLeft.x + cellWidth * cell.getPosition().y + cellWidth
                    / 2;
            float cy = topLeft.y + cellHeight * cell.getPosition().x
                    + cellHeight / 2;
            float radius = (cellWidth >= cellHeight) ? cellHeight
                    * Constants.RADIUS_MULTIPLIER : cellWidth
                    * Constants.RADIUS_MULTIPLIER;
            // Log.d("DrawUtil#drawCell", "cx: " + cx + " cy: " + cy +
            // " radius: "
            // + radius);
            canvas.drawCircle(cx, cy, radius, paint);
        }
    }

    public static void drawXCell(Cell cell, Canvas canvas, float cellWidth,
            float cellHeight, Paint paint, Point topLeft) {
        int color = cell.getColor();
        if (color != Color.WHITE) {
            paint.setColor(color);
            paint.setStrokeWidth(1);
            float x = topLeft.x + cellWidth * cell.getPosition().y;
            float y = topLeft.y + cellHeight * cell.getPosition().x;
            canvas.drawLine(x, y, x + cellWidth, y + cellHeight, paint);
            canvas.drawLine(x, y + cellHeight, x + cellWidth, y, paint);
        }
    }

    public static void drawField(Cell[] cells, Canvas canvas, Paint paint,
            Point topLeft, int rows, int columns) {
        float width = canvas.getWidth() - topLeft.x;
        float height = canvas.getHeight() - topLeft.y;
        Log.d("DrawUtil#drawField", "width: " + width + " height: " + height);

        float cellWidth = width / columns;
        float cellHeight = height / rows;

        Log.d("DrawUtil#drawField", "cellWidth: " + cellWidth + " cellHeight:"
                + cellHeight);
        for (int row = 0; row < rows; row++) {
            for (int col = 0; col < columns; col++) {
                drawCircleCell(cells[row * columns + col], canvas, cellWidth,
                        cellHeight, paint, topLeft);
                // drawXCell(field.getCells()[row * columns + col], canvas,
                // cellWidth,
                // cellHeight, paint, topLeft);
            }
        }
        // drawGrid(canvas, topLeft, rows, columns, width, height, cellWidth,
        // cellHeight);
    }

    private static void drawGrid(Canvas canvas, Point topLeft, int rows,
            int columns, float width, float height, float cellWidth,
            float cellHeight) {
        Paint p = new Paint();
        p.setAntiAlias(true);
        p.setColor(Color.BLUE);
        p.setStrokeWidth(0.2f);
        for (int row = 0; row < rows; row++) {
            canvas.drawLine(topLeft.x, topLeft.y + cellHeight * row, topLeft.x
                    + width, topLeft.y + cellHeight * row, p);
        }

        for (int column = 0; column < columns; column++) {
            canvas.drawLine(topLeft.x + cellWidth * column, topLeft.y,
                    topLeft.x + cellWidth * column, topLeft.y + height, p);
        }
    }
}
